import java.applet.Applet;
import java.applet.AppletStub;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Properties;

import javax.swing.JFrame;

/**
 * An Applet used for loading the RS3 Client.
 * 
 * @author Frosty Teh Snowman <Skype:travis.mccorkle>
 */
public class RS3Applet extends Applet implements AppletStub {

	/**
	 * An generated serial UID.
	 */
	private static final long serialVersionUID = 1670498001014004354L;

	/**
	 * Connecting to Rs or a Rsps
	 */
	public static boolean RSPS = true;
	public static boolean USING_ISAAC = !RSPS;

	/**
	 * Chooses ip based on if Rs or Rsps
	 */
	public static String HOST = RSPS ? "127.0.0.1" : "world2.runescape.com";
	public static int PORT = 43594;

	/**
	 * Constants
	 */
	public static int REVISION = 838;
	public static int SUB_REVISION = 1;
	
	/**
	 * Dump variables
	 */
	public static boolean DUMP_PACKETS = false;
	public static boolean DUMP_CONFIGS = false;
	public static boolean DUMP_SCRIPTS = false;
	public static boolean DUMP_INTERFACES = false;
	
	public static ArrayList<IncomingPacket> IDENTIFIED_PACKETS = new ArrayList<IncomingPacket>();
	
	static {
		IDENTIFIED_PACKETS.add(IncomingPacket.PING_PACKET);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_WORLD_LIST);
		IDENTIFIED_PACKETS.add(IncomingPacket.MINIMAP_FLAG);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_SKILL_LEVEL);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_INTERFACE);
		IDENTIFIED_PACKETS.add(IncomingPacket.CONFIG_1);
		IDENTIFIED_PACKETS.add(IncomingPacket.CONFIG_2);
		IDENTIFIED_PACKETS.add(IncomingPacket.GLOBAL_CONFIG_1);
		IDENTIFIED_PACKETS.add(IncomingPacket.GLOBAL_CONFIG_2);
		IDENTIFIED_PACKETS.add(IncomingPacket.SENDRUNSCRIPT);
		IDENTIFIED_PACKETS.add(IncomingPacket.ICOMPONENTSETTINGS);
		IDENTIFIED_PACKETS.add(IncomingPacket.GLOBALSTRING_1);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_WINDOW_PANE);
		IDENTIFIED_PACKETS.add(IncomingPacket.STRING_ON_ICOMPONENT);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_MESSAGE);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_FRIENDS_CHAT);
		IDENTIFIED_PACKETS.add(IncomingPacket.UPDATE_FRIENDS_LIST);
		IDENTIFIED_PACKETS.add(IncomingPacket.UPDATE_IGNORE_LIST);
		IDENTIFIED_PACKETS.add(IncomingPacket.RECIEVE_PM);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_PM);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_FRIENDS_CHAT_MESSAGE);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_SOUND_PACKET);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_COORDS);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_GROUND_ITEM);
		IDENTIFIED_PACKETS.add(IncomingPacket.REMOVE_GROUND_ITEM);
		IDENTIFIED_PACKETS.add(IncomingPacket.PLAYER_UPDATE_PACKET);
		IDENTIFIED_PACKETS.add(IncomingPacket.LARGE_NPC_UPDATE);
		IDENTIFIED_PACKETS.add(IncomingPacket.NPC_UPDATE);
		IDENTIFIED_PACKETS.add(IncomingPacket.BOT_CHECK_PACKET_REFLECTION);
		IDENTIFIED_PACKETS.add(IncomingPacket.CAMERA_BASE_COORD_RELATED); //TODO
		IDENTIFIED_PACKETS.add(IncomingPacket.PLAYER_OPTION_PACKET);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_ITEMS_PACKET);
		IDENTIFIED_PACKETS.add(IncomingPacket.BASE_COORDINATE_RELATED);
		IDENTIFIED_PACKETS.add(IncomingPacket.CLOSE_INTERFACE);
		IDENTIFIED_PACKETS.add(IncomingPacket.ANIMATE_ICOMPONENT);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_MAP_REGION);
		IDENTIFIED_PACKETS.add(IncomingPacket.PLAYER_ON_ICOMPONENT);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_DYNAMIC_MAP_REGION);
		IDENTIFIED_PACKETS.add(IncomingPacket.NPC_ON_ICOMPONENT);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_RUN_ENERGY);
		IDENTIFIED_PACKETS.add(IncomingPacket.HIDE_ICOMPONENT);
		IDENTIFIED_PACKETS.add(IncomingPacket.GLOBALSTRING_2);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_UPDATE_ITEMS);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_WEIGHT);
		IDENTIFIED_PACKETS.add(IncomingPacket.REFRESH_NIS);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_DYNAMIC_MAP_REGION);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_DYNAMIC_MAP_REGION);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_DYNAMIC_MAP_REGION);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_DYNAMIC_MAP_REGION);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_DYNAMIC_MAP_REGION);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_DYNAMIC_MAP_REGION);
		IDENTIFIED_PACKETS.add(IncomingPacket.SEND_DYNAMIC_MAP_REGION);
	}

	/**
	 * The parameters of the client.
	 */
	private Properties client_parameters = new Properties();

	/**
	 * The current frame of the client application.
	 */
	public JFrame clientFrame = null;

	/**
	 * The main entry point of the current application.
	 * 
	 * @param args
	 *            The command line arguments.
	 * @throws IOException
	 * @throws MalformedURLException
	 */
	public static void main(String... args) throws MalformedURLException, IOException {
		RS3Applet rs3applet = new RS3Applet();
		rs3applet.readParameters();
		rs3applet.openFrame();
		rs3applet.startClient();
		rs3applet.clientFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	/**
	 * Starts the actual client.
	 */
	private void startClient() {
		try {
			client clientt = new client();

			clientt.supplyApplet(this);
			clientt.init();
			clientt.start();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Opens the actual frame application.
	 */
	private void openFrame() {
		clientFrame = new JFrame("Jagex 838");
		clientFrame.add(this);
		clientFrame.setVisible(true);
		clientFrame.setSize(800, 600);
	}

	/**
	 * Reads the parameters text file, and stores the parameters.
	 * 
	 * @throws IOException
	 * @throws MalformedURLException
	 */
	private void readParameters() throws MalformedURLException, IOException {
		if (!RSPS) {
			for (String line : fetchPageDetails()) {
				if (line.contains("<param name=")) {
					String key = line.split("<param name=\"")[1].split("\" ")[0];
					String value = line.split("value=\"")[1].split("\">'")[0];
					if (value.isEmpty())
						value = "";
					client_parameters.put(key, value);
				}
			}
		} else {
			client_parameters.put("minor", "1");
			client_parameters.put("java_arguments", "-Xmx384m -Xss2m -Dsun.java2d.noddraw=true -XX:CompileThreshold=1500 -Xincgc -XX:+UseConcMarkSweepGC -XX:+UseParNewGC");
			client_parameters.put("haveie6", "true");
			client_parameters.put("gamepack", "gamepackJq2i/QKHhmq4jLwSV1fFLUtKNDwK0rp0_4709628.jar");
			client_parameters.put("major", "838");
			client_parameters.put("centerimage", "true");
			client_parameters.put("10", "0");
			client_parameters.put("11", "0");
			client_parameters.put("image", "http://www.runescape.com/img/game/splash6.gif");
			client_parameters.put("12", "");
			client_parameters.put("13", "3738");
			client_parameters.put("14", "true");
			client_parameters.put("-1", "yOCVtXTLJ5WIbJtJbv-Zvw");
			client_parameters.put("15", "1101");
			client_parameters.put("16", "");
			client_parameters.put("18", "false");
			client_parameters.put("0", "qjqFkPzASOjuTJlWSeZDsQ");
			client_parameters.put("1", "halign=true|valign=true|image=rs_logo.gif,0,-43|rotatingimage=rs3_loading_spinner.gif,0,47,9.6|progress=true,Verdana,13,0xFFFFFF,0,51");
			client_parameters.put("2", "false");
			client_parameters.put("3", "false");
			client_parameters.put("4", "false");
			client_parameters.put("5", "false");
			client_parameters.put("6", "0");
			client_parameters.put("7", "false");
			client_parameters.put("8", "false");
			client_parameters.put("name", "TitanPK");
			client_parameters.put("9", "true");
			client_parameters.put("20", "1101");
			client_parameters.put("21", "Jq2i/QKHhmq4jLwSV1fFLUtKNDwK0rp0");
			client_parameters.put("22", "1236637085");
			client_parameters.put("23", "0");
			client_parameters.put("24", "0");
			client_parameters.put("26", "");
			client_parameters.put("28", "0");
			client_parameters.put("29", "");
			client_parameters.put("separate_jvm", "true");
			client_parameters.put("30", "0");
			client_parameters.put("31", "0");
			client_parameters.put("32", "225");
			client_parameters.put("33", HOST);
			client_parameters.put("34", HOST);
			client_parameters.put("35", "");
			client_parameters.put("36", ".runescape.com");
			client_parameters.put("37", HOST);
			client_parameters.put("38", "7");
			client_parameters.put("39", "wwGlrZHF5gKN6D3mDdihco3oPeYN2KFybL9hUUFqOvk");
			client_parameters.put("boxbgcolor", "black");
			client_parameters.put("boxborder", "false");
			client_parameters.put("40", HOST);
			client_parameters.put("41", "http://services.runescape.com/m=gamelogspecs/clientstats?data=");
			client_parameters.put("42", "false");
			client_parameters.put("43", "");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletStub#appletResize(int, int)
	 */
	@Override
	public void appletResize(int dimensionX, int dimensionY) {
		super.resize(new Dimension(dimensionX, dimensionY));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.Applet#getParameter(java.lang.String)
	 */
	@Override
	public String getParameter(String paramName) {
		return (String) client_parameters.get(paramName);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.Applet#getDocumentBase()
	 */
	@Override
	public URL getDocumentBase() {
		try {
			return new URL("http://" + HOST);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.Applet#getCodeBase()
	 */
	@Override
	public URL getCodeBase() {
		try {
			return new URL("http://" + HOST);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Fetches the current parameters from the specified address
	 * 
	 * @return
	 * @throws MalformedURLException
	 * @throws IOException
	 */
	public static ArrayList<String> fetchPageDetails() throws MalformedURLException, IOException {
		ArrayList<String> pageSource = new ArrayList<String>();
		BufferedReader reader = new BufferedReader(new InputStreamReader(new URL("http://" + HOST).openStream()));
		String line;
		while ((line = reader.readLine()) != null)
			pageSource.add(line);
		reader.close();
		return pageSource;
	}

}